uniform sampler2D u_texture;
uniform mediump vec3 vHSV;

varying mediump vec2 v_texCoords;
varying mediump vec4 v_color;

mediump vec3 rgb2hsv(mediump vec3 c);
mediump vec3 hsv2rgb(mediump vec3 c);

void main()
{
  mediump vec4 originalColor = texture2D(u_texture, v_texCoords);
  if(originalColor.w==0.0)
  {
      gl_FragColor = originalColor;
  }
  else{
    mediump vec3 fragRGB = originalColor.rgb;
    mediump vec3 fragHSV = rgb2hsv(fragRGB).xyz;
    fragHSV.x += vHSV.x;
    fragHSV.x = mod(fragHSV.x,1.0);
    fragHSV.yz *= vHSV.yz;
    fragRGB = hsv2rgb(fragHSV);
    gl_FragColor = vec4(fragRGB, originalColor.w);
  }
}

mediump vec3 rgb2hsv(mediump vec3 c)
{
    mediump vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    mediump vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    mediump vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    mediump float d = q.x - min(q.w, q.y);
    mediump float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

mediump vec3 hsv2rgb(mediump vec3 c)
{
    mediump vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    mediump vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}